/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include <glu.h>
#include "canvas.h"

Canvas::Canvas( QWidget *parent )
		: QGLWidget( parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer|QGL::NoDepthBuffer ), parent )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

	makeCheckBoardImage( );
}

void Canvas::makeCheckBoardImage( )
{
	int i, j, c;

	for( i=0; i<IMGWIDTH; i++ ) {
		for( j=0; j<IMGHEIGHT; j++ ) {
			c = (  ((i&0x8)==0)^((j&0x8)==0) )*255;
			checkImage[i][j][0] = (GLubyte) c;
			checkImage[i][j][1] = (GLubyte) c;
			checkImage[i][j][2] = (GLubyte) c;
			checkImage[i][j][3] = (GLubyte) 255;
		}
	}
}


void Canvas::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Canvas::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Canvas::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Canvas::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	gluPerspective( 40.0, aspect, 1.0, 30.0 );
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */
	glTranslatef( 0.0, 0.0, -3.6 );

	emit changeSize ( );
}

QSizePolicy Canvas::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Canvas::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix( );

	glRotatef ( xrot , 1.0, 0.0, 0.0);
	glRotatef ( yrot , 0.0, 1.0, 0.0);


	glRotatef ( angle , 0.0, 0.0, 1.0);

	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	// glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND );
	glBindTexture( GL_TEXTURE_2D, texName );
	glBegin( GL_QUADS );
		glColor3f( 0.0, 0.0, 1.0 );
		glTexCoord2f( 0.0, 0.0 );
		glVertex3f( -1.0, -1.0, 0.0 );
		  
		glTexCoord2f( 0.0, 1.0 );
		glVertex3f( -1.0,  1.0, 0.0 );

		glTexCoord2f( 1.0, 1.0 );
		glVertex3f( 1.0,  1.0, 0.0 );

		glTexCoord2f( 1.0, 0.0 );
		glVertex3f( 1.0, -1.0, 0.0 );
	glEnd( );
	glPopMatrix( );

	// swapBuffers( );
}

void Canvas::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluOrtho2D(-1.0, 1.0, -1.0, 1.0 );

	glEnable (GL_DEPTH_TEST);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	/** Para la textura **/
	glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	glGenTextures( 1, &texName );
	glBindTexture( GL_TEXTURE_2D, texName );

	/** Que hacer si las coordenadas de textura u  v > 1.0 **/
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT ); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT ); 

	/** Que filtros se aplican a la imagen de textura para visualizarla **/
	// glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST ); 
	// glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST ); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR ); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR ); 

	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, IMGWIDTH, IMGHEIGHT, 0, GL_RGBA,
	                  GL_UNSIGNED_BYTE, checkImage );

	glEnable( GL_TEXTURE_2D );

}


